; main.s
; Runs on LM3S1968
; Provide functions that initialize a GPIO pin and turn it on and off.
; Use bit-banded I/O.
; Daniel Valvano
; February 15, 2012

;  This example accompanies the book
;  "Embedded Systems: Introduction to the Arm Cortex M3",
;  ISBN: 978-1469998749, Jonathan Valvano, copyright (c) 2012
;  Program 4.3
;
;Copyright 2012 by Jonathan W. Valvano, valvano@mail.utexas.edu
;   You may use, edit, run or distribute this file
;   as long as the above copyright notice remains
;THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
;OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
;MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
;VALVANO SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
;OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
;For more information about my classes, my research, and my books, see
;http://users.ece.utexas.edu/~valvano/

; solid state relay connected to PG2

GPIO_PORTG2        EQU 0x40026010
GPIO_PORTG7        EQU 0x40026200
GPIO_PORTG_DATA_R  EQU 0x400263FC
GPIO_PORTG_DIR_R   EQU 0x40026400
GPIO_PORTG_AFSEL_R EQU 0x40026420
GPIO_PORTG_PUR_R   EQU 0x40026510
GPIO_PORTG_DEN_R   EQU 0x4002651C
SYSCTL_RCGC2_R     EQU 0x400FE108
SYSCTL_RCGC2_GPIOG EQU 0x00000040   ; port G Clock Gating Control

        AREA    |.text|, CODE, READONLY, ALIGN=2
        THUMB
        EXPORT  Start
Start
    BL  SSR_Init                    ; initialize PG2 and make it output
    ; set direction register
    LDR R1, =GPIO_PORTG_DIR_R       ; R1 = &GPIO_PORTG_DIR_R
    LDR R0, [R1]                    ; R0 = [R1]
    BIC R0, R0, #0x80               ; R0 = R0&~0x80 (make PG7 input; PG7 built-in "Select" button) (default setting)
    STR R0, [R1]                    ; [R1] = R0
    ; regular port function
    LDR R1, =GPIO_PORTG_AFSEL_R     ; R1 = &GPIO_PORTG_AFSEL_R
    LDR R0, [R1]                    ; R0 = [R1]
    BIC R0, R0, #0x80               ; R0 = R0&~0x80 (disable alt funct on PG7) (default setting)
    STR R0, [R1]                    ; [R1] = R0
    ; pull-up resistors on switch pins
    LDR R1, =GPIO_PORTG_PUR_R       ; R1 = &GPIO_PORTG_PUR_R
    LDR R0, [R1]                    ; R0 = [R1]
    ORR R0, R0, #0x80               ; R0 = R0|0x80 (enable pull-up on PG7) (default setting on LM3S811, not default on other microcontrollers)
    STR R0, [R1]                    ; [R1] = R0
    ; enable digital port
    LDR R1, =GPIO_PORTG_DEN_R       ; R1 = &GPIO_PORTG_DEN_R
    LDR R0, [R1]                    ; R0 = [R1]
    ORR R0, R0, #0x80               ; R0 = R0|0x80 (enable digital I/O on PG7) (default setting on LM3S811, not default on other microcontrollers)
    STR R0, [R1]                    ; [R1] = R0
    LDR R4, =GPIO_PORTG7            ; R4 = &GPIO_PORTG7
loop                                ; in this loop, the appliance (PG2) is on when the button is released
    BL  SSR_On
spin1                               ; proceed only when the button is pressed
    LDR R0, [R4]                    ; R0 = [R4] (read status of PG7)
    CMP R0, #0x80                   ; R0 == 0x80?
    BEQ spin1                       ; if so, spin
    BL  SSR_Off
spin2                               ; proceed only when the button is released
    LDR R0, [R4]                    ; R0 = [R4] (read status of PG7)
    CMP R0, #0x80                   ; R0 != 0x80?
    BNE spin2                       ; if so, spin
    B   loop

;------------SSR_Init------------
; Make PG2 an output and enable digital I/O.
; Input: none
; Output: none
; Modifies: R0, R1
SSR_Init
    ; activate clock for Port G
    LDR R1, =SYSCTL_RCGC2_R         ; R1 = &SYSCTL_RCGC2_R
    LDR R0, [R1]                    ; R0 = [R1]
    ORR R0, R0, #SYSCTL_RCGC2_GPIOG ; R0 = R0|SYSCTL_RCGC2_GPIOG
    STR R0, [R1]                    ; [R1] = R0
    NOP
    NOP                             ; allow time to finish activating
    ; set direction register
    LDR R1, =GPIO_PORTG_DIR_R       ; R1 = &GPIO_PORTG_DIR_R
    LDR R0, [R1]                    ; R0 = [R1]
    ORR R0, R0, #0x04               ; R0 = R0|0x04 (make PG2 output)
    STR R0, [R1]                    ; [R1] = R0
    ; regular port function
    LDR R1, =GPIO_PORTG_AFSEL_R     ; R1 = &GPIO_PORTG_AFSEL_R
    LDR R0, [R1]                    ; R0 = [R1]
    BIC R0, R0, #0x04               ; R0 = R0&~0x04 (disable alt funct on PG2) (default setting)
    STR R0, [R1]                    ; [R1] = R0
    ; enable digital port
    LDR R1, =GPIO_PORTG_DEN_R       ; R1 = &GPIO_PORTG_DEN_R
    LDR R0, [R1]                    ; R0 = [R1]
    ORR R0, R0, #0x04               ; R0 = R0|0x04 (enable digital I/O on PG2) (default setting on LM3S811, not default on other microcontrollers)
    STR R0, [R1]                    ; [R1] = R0
    BX  LR                          ; return

;------------SSR_On------------
; Make PG2 high.
; Input: none
; Output: none
; Modifies: R0, R1
SSR_On
    LDR R1, =GPIO_PORTG2            ; R1 = &GPIO_PORTG2
    MOV R0, #0x04                   ; R0 = 0x04 (turn on the appliance)
    STR R0, [R1]                    ; [R1] = R0, write to PG2
    BX  LR                          ; return

;------------SSR_Off------------
; Make PG2 low.
; Input: none
; Output: none
; Modifies: R0, R1
SSR_Off
    LDR R1, =GPIO_PORTG2            ; R1 = &GPIO_PORTG2
    MOV R0, #0x00                   ; R0 = 0x00 (turn off the appliance)
    STR R0, [R1]                    ; [R1] = R0, write to PG2
    BX  LR                          ; return

    ALIGN                           ; make sure the end of this section is aligned
    END                             ; end of file
